/*
 * Copyright (c) 2016, 2017, 2018, 2019 FabricMC
 * Copyright 2022 The Quilt Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.fabricmc.fabric.api.registry;

import java.util.Objects;
import net.minecraft.class_1792;
import net.minecraft.class_1812;
import net.minecraft.class_1842;
import net.minecraft.class_1845;
import net.minecraft.class_1856;

/**
 * Counterpart of {@link class_1845} with methods that allow adding recipes which use Ingredients instead of Items.
 */
public final class FabricBrewingRecipeRegistry {
	private FabricBrewingRecipeRegistry() {
	}

	/**
	 * Register a recipe for brewing one potion type into another (e.g. regular to splash).
	 * Only one recipe is necessary for all potions of the input type to be brewable into the output type using the ingredient.
	 * Use {@link class_1845#method_8080(class_1792)} to register new potion types.
	 * @param input the input potion type (e.g. regular potion)
	 * @param ingredient the required ingredient (e.g. gunpowder)
	 * @param output the output type (e.g. splash potion)
	 * @see class_1845#method_8071(class_1792, class_1792, class_1792)
	 */
	public static void registerItemRecipe(class_1812 input, class_1856 ingredient, class_1812 output) {
		Objects.requireNonNull(input, "Input cannot be null!");
		Objects.requireNonNull(ingredient, "Ingredient cannot be null!");
		Objects.requireNonNull(output, "Output cannot be null!");

		class_1845.field_8959.add(new class_1845.class_1846<>(input, ingredient, output));
	}

	/**
	 * Register a recipe for converting from one potion to another (e.g. awkward to instant health).
	 * This does not automatically create long or strong versions of the output potion.
	 * They require separate recipes.
	 * @param input input potion (e.g. awkward)
	 * @param ingredient the required ingredient (e.g. glistering melon)
	 * @param output output potion (e.g. instant health)
	 * @see class_1845#method_8074(class_1842, class_1792, class_1842)
	 */
	public static void registerPotionRecipe(class_1842 input, class_1856 ingredient, class_1842 output) {
		Objects.requireNonNull(input, "Input cannot be null!");
		Objects.requireNonNull(ingredient, "Ingredient cannot be null!");
		Objects.requireNonNull(output, "Output cannot be null");

		class_1845.field_8956.add(new class_1845.class_1846<>(input, ingredient, output));
	}
}
