/*
 * Copyright (c) 2016, 2017, 2018, 2019 FabricMC
 * Copyright 2022 The Quilt Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.fabricmc.fabric.impl.content.registry;

import java.util.IdentityHashMap;
import java.util.Map;

import org.quiltmc.qsl.item.content.registry.api.ItemContentRegistries;
import org.quiltmc.quilted_fabric_api.impl.content.registry.util.QuiltDeferringQueues;
import net.fabricmc.fabric.api.registry.FuelRegistry;
import net.minecraft.class_1792;
import net.minecraft.class_1935;
import net.minecraft.class_6862;

// Fabric To-Do: Clamp values to 32767 (+ add hook for mods which extend the limit to disable the check?)
// About that, Quilt's equivalent API supports integer values properly, no need for that
public final class FuelRegistryImpl implements FuelRegistry {
	//private static final Logger LOGGER = LoggerFactory.getLogger(FuelRegistryImpl.class);

	public FuelRegistryImpl() { }

	public Map<class_1792, Integer> getFuelTimes() {
		Map<class_1792, Integer> ret = new IdentityHashMap<>();

		for (var entry : ItemContentRegistries.FUEL_TIMES) {
			ret.put(entry.entry(), entry.value());
		}

		return ret;
	}

	@Override
	public Integer get(class_1935 item) {
		return ItemContentRegistries.FUEL_TIMES.getNullable(item.method_8389());
	}

	@Override
	public void add(class_1935 item, Integer cookTime) {
		QuiltDeferringQueues.addEntryWithItemConvertible(ItemContentRegistries.FUEL_TIMES, item, cookTime.intValue());
	}

	@Override
	public void add(class_6862<class_1792> tag, Integer cookTime) {
		ItemContentRegistries.FUEL_TIMES.put(tag, cookTime.intValue());
	}

	@Override
	public void remove(class_1935 item) {
		ItemContentRegistries.FUEL_TIMES.put(item.method_8389(), 0);
	}

	@Override
	public void remove(class_6862<class_1792> tag) {
		ItemContentRegistries.FUEL_TIMES.put(tag, 0);
	}

	@Override
	public void clear(class_1935 item) {
		ItemContentRegistries.FUEL_TIMES.remove(item.method_8389());
	}

	@Override
	public void clear(class_6862<class_1792> tag) {
		ItemContentRegistries.FUEL_TIMES.remove(tag);
	}

	public void apply(Map<class_1792, Integer> map) { }

	public void resetCache() { }
}
